// -------------------------------------------------------------------
// MAdLib - Copyright (C) 2008-2009 Universite catholique de Louvain
//
// See the Copyright.txt and License.txt files for license information. 
// You should have received a copy of these files along with MAdLib. 
// If not, see <http://www.madlib.be/license/>
//
// Please report all bugs and problems to <contrib@madlib.be>
//
// Authors: Gaetan Compere, Jean-Francois Remacle
// -------------------------------------------------------------------

#include "SizeFieldBase.h"
#include "MAdOutput.h"
#include "MathUtils.h"

using std::string;

namespace MAd {

  // -------------------------------------------------------------------
  // Length squared computation
  // -------------------------------------------------------------------

  // -------------------------------------------------------------------
  double SizeFieldBase::SF_E_lengthSq(const pEdge edge) const
  {
    return SF_VV_lengthSq(E_vertex(edge,0),E_vertex(edge,1));
  }

  // -------------------------------------------------------------------
  // Center of edge computation
  // -------------------------------------------------------------------

  // -------------------------------------------------------------------
  double SizeFieldBase::SF_XYZ_center(const double xyz[2][3], const pMSize pS[2],
                                      double center[3], pMSize * cSize) const
  {
    if( !pS[0] || !pS[1] ) {
      printf("Error in SizeFieldBase::SF_XYZ_center: one of the sizes is NULL\n");
      throw;
    }

    double vec[3];
    diffVec(xyz[1],xyz[0],vec);

    double lenSq[2];
    lenSq[0] = pS[0]->lengthSqInDir(vec);
    lenSq[1] = pS[1]->lengthSqInDir(vec);

    double cParam = 1. / ( 1. + pow(lenSq[1]/lenSq[0],0.25) );

    for (int i=0; i<3; i++) {
      center[i] = cParam * xyz[1][i] + (1-cParam) * xyz[0][i];
    }

    *cSize = MS_interpolate(pS[0],pS[1],cParam);

    return cParam;
  }

  // -------------------------------------------------------------------
  void SizeFieldBase::printPos(const pMesh mesh, const string name)
  {
    MAdGmshOutput(mesh, this, name.c_str(), OD_ISOSIZEFIELD);
  }

  // -------------------------------------------------------------------

}
