// -------------------------------------------------------------------
// MAdLib - Copyright (C) 2008-2009 Universite catholique de Louvain
//
// See the Copyright.txt and License.txt files for license information. 
// You should have received a copy of these files along with MAdLib. 
// If not, see <http://www.madlib.be/license/>
//
// Please report all bugs and problems to <contrib@madlib.be>
//
// Authors: Gaetan Compere, Jean-Francois Remacle
// -------------------------------------------------------------------

#include "CPUTimeManager.h"

#ifdef PARALLEL
#include <mpi.h>
#else
#ifndef _WIN_
#include <sys/time.h>
#else
#include <time.h>
#include <windows.h> 
#if defined(_MSC_VER) || defined(_MSC_EXTENSIONS)
#define DELTA_EPOCH_IN_MICROSECS  11644473600000000Ui64
#else
#define DELTA_EPOCH_IN_MICROSECS  11644473600000000ULL
#endif
 
struct timezone 
{
  int  tz_minuteswest; /* minutes W of Greenwich */
  int  tz_dsttime;     /* type of dst correction */
};
 
int get_time_of_day(struct timeval *tv, struct timezone *tz)
{
  FILETIME ft;
  unsigned __int64 tmpres = 0;
  static int tzflag;
 
  if (NULL != tv)
    {
      GetSystemTimeAsFileTime(&ft);
 
      tmpres |= ft.dwHighDateTime;
      tmpres <<= 32;
      tmpres |= ft.dwLowDateTime;
 
      // converting file time
      tmpres -= DELTA_EPOCH_IN_MICROSECS; 
      tmpres /= 10;  // convert into microseconds
      tv->tv_sec = (long)(tmpres / 1000000UL);
      tv->tv_usec = (long)(tmpres % 1000000UL);
    }
 
  if (NULL != tz)
    {
      if (!tzflag)
        {
          _tzset();
          tzflag++;
        }
      tz->tz_minuteswest = _timezone / 60;
      tz->tz_dsttime = _daylight;
    }
 
  return 0;
}
#endif

#endif
#ifdef PARALLEL
#include "mpi.h"
#endif

#ifdef PARALLEL
#include <mpi.h>
#endif

// -------------------------------------------------------------------

namespace MAd {

  // -------------------------------------------------------------------
  void CPUTimeManager::initialize()
  {
  }

  // -------------------------------------------------------------------
  void CPUTimeManager::finalize()
  {
  }

  // -------------------------------------------------------------------
  double CPUTimeManager::getTime() const {
  
#ifdef PARALLEL
    return MPI_Wtime();
#else
    struct timeval tp;
    struct timezone tz;
#ifdef _WIN_
    get_time_of_day(&tp,&tz);
#else
    gettimeofday(&tp,&tz);
#endif

    return ((double) tp.tv_sec +
            (double) ((double) .000001 * (double) tp.tv_usec));
#endif
  }

  // -------------------------------------------------------------------
  double CPUTimeManager::elapsedTime() const {

    return ( getTime() - initialTime );

  }
}

// -------------------------------------------------------------------
