// -------------------------------------------------------------------
// MAdLib - Copyright (C) 2008-2009 Universite catholique de Louvain
//
// See the Copyright.txt and License.txt files for license information. 
// You should have received a copy of these files along with MAdLib. 
// If not, see <http://www.madlib.be/license/>
//
// Please report all bugs and problems to <contrib@madlib.be>
//
// Authors: Gaetan Compere, Jean-Francois Remacle
// -------------------------------------------------------------------

#include "MSops.h"
#include "AdaptInterface.h"
#include "NullSField.h"
#include "AnalyticalSField.h"

#include <iostream>
using std::cout;
#include <sys/resource.h>
#include <string>
using std::string;

using namespace MAd;

#ifdef PARALLEL
#include "mpi.h"
#endif

// ----------------------------------------------------------------------
int main(int argc, char* argv[]) {

#ifdef PARALLEL 
  MPI_Init(&argc, &argv);
#endif

  // Check input
  // ------------
  if ( argc != 3 && argc != 2  ) {
    printf("Error: usage: \'executable mshFile [geoFile]\'\n");
    exit(0);
  }
  string meshFile = argv[1];

  // Build tools
  // ------------
  printf ("Analyzing mesh %s...\n\n",meshFile.c_str());

  // --- Reading model ---
  pGModel model = 0;
  GM_create(&model,"theModel");
  
  if ( argc == 3 ) {
    string geoFile  = argv[2];
    GM_readFromGEO(model, geoFile.c_str());
  }
  else {
    GM_readFromMSH(model, meshFile.c_str());
  }

  pMesh mesh = M_new(model);
  M_load(mesh,meshFile.c_str());
  SizeFieldBase * sizeField = new AnalyticalSField("1.");
  
  MeshAdapter* ma = new MeshAdapter(mesh,sizeField);

  // temporary info
//   int nbVolEdges = 0;
//   int dim = M_dim(mesh);
//   int numR[20];
//   for (int i=0; i<20; i++) numR[i]=0;
//   EIter ei = M_edgeIter(mesh);
//   while ( pEdge edge = EIter_next(ei) ) {
//     if ( E_whatInType(edge) == dim ) {
//       int num = E_numRegions(edge);
//       numR[num] = numR[num]++;
//       nbVolEdges++;
//     }
//   }
//   EIter_delete(ei);
//   printf("Num points in edges crown:\n");
//   printf("Edges: %d\n",nbVolEdges);
  
//   for (int i=0; i<20; i++) {
//     double ratio = ( (double) (numR[i]) ) / ( (double)nbVolEdges );
//     printf("%d\t%d\t%f\n",i,numR[i],ratio);
//   }
//   printf("\n");


  // Outputs
  // --------
  ma->printStatistics(std::cout);
  ma->writePos("meanRatio.pos",OD_MEANRATIO);

  // Cleaning
  // ---------
  if (ma) delete ma;
  if (sizeField) delete sizeField;

#ifdef PARALLEL 
  MPI_Finalize();
#endif
}

// ----------------------------------------------------------------------
