// -------------------------------------------------------------------
// MAdLib - Copyright (C) 2008-2009 Universite catholique de Louvain
//
// See the Copyright.txt and License.txt files for license information. 
// You should have received a copy of these files along with MAdLib. 
// If not, see <http://www.madlib.be/license/>
//
// Please report all bugs and problems to <contrib@madlib.be>
//
// Authors: Koen Hillewaert
// -------------------------------------------------------------------

#include "MeshDataBaseCommCheck.h"
#include "MeshDataBaseMessage.h"
#include "MeshDataBaseInterface.h"

#include <sstream>
using std::ostringstream;
using std::ostream;
#include <iostream>

namespace MAd {

  // -----------------------------------------------------------------------------

  MDB_CommCheck::MDB_CommCheck():numRecvs(0),numSends(0),numErrors(0) {}

  // -----------------------------------------------------------------------------

  void MDB_CommCheck::printStatus(ostream& out) const {

    out << "Number of sends " << numSends
        << ", number of receives " << numRecvs
        << ", number of errors " << numErrors << std::endl;
  }


  // -----------------------------------------------------------------------------

  void* MDB_CommCheck::sendData(pEntity pe,int iProc,int& size) {  
  
    int type = EN_type(pe);
    void* buf = NULL;
    size = 0;
  
    switch (type) {
    
    case 0:
      {
        size += 2*sizeof(int);
        buf = malloc(size);

        int* ibuf = reinterpret_cast<int*>(buf);

        *(ibuf++) = 1;
        *(ibuf++) = EN_id(pe);

        return buf;
      
        break;
      }
    case 1:
      {
        pEdge pl = (pEdge) pe;
        size += (1 + 2) * sizeof(int);
        buf = malloc(size);

        int *ibuf = reinterpret_cast<int*>(buf);

        *(ibuf++) = 2;
        *(ibuf++) = EN_id((pEntity) E_vertex(pl,0));
        *(ibuf++) = EN_id((pEntity) E_vertex(pl,1));      
        break;
      }
    case 2:
      {
      
        pFace pf = (pFace) pe;
        size = (1 + F_numVertices(pf)) * sizeof(int);
        buf  = malloc(size);

        int *ibuf = reinterpret_cast<int*>(buf);

        *(ibuf++) = F_numVertices(pf);
        for (int i=0;i<F_numVertices(pf);i++) {
          *(ibuf++) = EN_id((pEntity) F_vertex(pf,i));
        }
        break;
      }
    }
    return buf;
  }

  // -----------------------------------------------------------------------------

  void MDB_CommCheck::receiveData(pEntity pe,int from,void* buf) {
  
    int type = EN_type(pe);

    int* ibuf = reinterpret_cast<int*>(buf);
    int  np   = *(ibuf++);
  
    switch (type) {

    case 0: 
      {
        int id = *(ibuf++);
        if (id != EN_id(pe)) {
          Msg_char(MDB_FATAL,
                   "Non-corresponding communication vertex from proc %d : %d vs %d\n",
                   from,EN_id(pe),id);
        }
        break;
      }
    case 1:
      {
        pEdge pl = (pEdge) pe;
        bool check = true;
      
        int idSend[2];
        int idRecv[2];
      
        for (int i=0;i<2;i++){
          idSend[i] = *(ibuf++);
          idRecv[i] = EN_id((pEntity) E_vertex(pl,i));
          if (idSend[i] != idRecv[i]) check = false;
        }

        if (check == false) {
          Msg_char(MDB_FATAL,
                   "Non-corresponding edge from proc %d : (%d,%d) vs (%d,%d)\n",
                   from,idSend[0],idSend[1],idRecv[0],idRecv[1]);
        }
        break;
      }
    case 2:
      {
        pFace pf = (pFace) pe;
        bool check = (np == F_numVertices(pf));
      
        int idSend[4];
        int idRecv[4];
      
        for (int i=0;i<np ;i++){
          idSend[i] = *(ibuf++);
          idRecv[i] = EN_id((pEntity) F_vertex(pf,i));
          if (idSend[i] != idRecv[i]) check = false;
        }

        if (check == false) {
          
          ostringstream recv;
          for (int i=0;i<np;i++) recv << " " << idSend[i];
          ostringstream send;
          for (int i=0;i<F_numVertices(pf);i++) send << " " << idRecv[i];
        
          Msg_char(MDB_FATAL,
                   "Non-corresponding face from proc %d - (%s) vs (%s)\n",
                   recv.str().c_str(),send.str().c_str());
        }
        break;
      } 
    }
  }

  // -----------------------------------------------------------------------------

}
